import { Inject, Injectable } from '@nestjs/common';
import { ClientProxy } from '@nestjs/microservices';
import { interval, Observable, Subject } from 'rxjs';
import * as net from 'net'
import { ConfigService } from '@nestjs/config';
import { connectToAfisJava } from 'libs/java/afis.utils';
import { FpVerificationPayload, FpVerificationResponse } from 'libs';
import { v1 as uuid } from 'uuid'

@Injectable()
export class FisVerificationService {
  private port: number
  private host: string
  private javaClient: net.Socket
  private incomingMessageFromJava!: Subject<FpVerificationResponse>
  private fpTemplate: any[] = []

  constructor(
    @Inject(`SAMPLEAPP_SERVICE`) private sampleClient: ClientProxy,
    private configService: ConfigService
  ) {
    this.port = this.configService.get<number>('afis.tcpPort') as number
    this.host = this.configService.get<string>('afis.host') as string

    this.javaClient = connectToAfisJava(this.host, this.port)
    this.javaClient.on(`data`, (data) => {
      let message: FpVerificationResponse = JSON.parse(data.toString())
      console.log(message)
      // this.incomingMessageFromJava.next(message)
    })
  }

  public handleMessage(message: any): void {
    let data = JSON.parse(message)
    // console.log(data)
    let extractionMessage: FpVerificationPayload = {
      id: uuid(),
      cmd: `Registration`,
      date: new Date(),
      data: data.messageData,
      fpTemplateArray: [],
      message: `LOLz`
    }
    this.javaClient.write(JSON.stringify(extractionMessage) + `\n`)
  }

}

/* Reason based on ChatGPT: Why ClientProxy.send() or emit() won't work 
The NestJS ClientProxy (when configured for Transport.TCP) uses its own custom binary protocol under the hood. This includes:
A message envelope that wraps your pattern and data.
Message IDs and event headers.
A specific serialization/deserialization logic that is not plain JSON.
NO newline character at the end.
So when your Java server, which expects a newline-terminated JSON message, receives this custom binary format, it gets confused or sees null.*/

/* data: {
messageData: string
messageTemplateData: string
deviceInfo: object} */